/*
 * sdl_app2d.hpp
 *
 * SdlApp2D: App2D using SDL
 * Note that timers are rounded up to the next 10 ms with SDL !
 *
 * Once app is started SDL can be accessed via SDL_GetVideoSurface() in the normal way.
 * (The app creates a double buffered display so SDL_Flip(SDL_GetVideoSurface()) should be 
 * called after each frame.)
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef SDL_APP_2D_HPP
#define SDL_APP_2D_HPP

#include "app2d.hpp"

#include <SDL.h>

#include <map>
#include <vector>
using namespace std;


//
// SdlApp2D
//

class SdlApp2D : public App2D {
public:
    // Initialization.
    SdlApp2D();

    // Dtor frees all resources and closes any open display.
    virtual ~SdlApp2D();

    // Full Screen Displays
    virtual int findDisplayModes();
    virtual const DisplayMode & getDisplayMode(int n);
    virtual void openFullScreen(int mode_num, const char *title);

    // Windowed Displays
    virtual void openWindowed(int x, int y, int width, int height,
                              const char *title);

    // Query functions for displays
    virtual int getDisplayWidth();
    virtual int getDisplayHeight();
    virtual bool isFullScreen();
    virtual bool isWindowed();
    virtual bool hasFocus();

    // Main Loop
    virtual void enterMainLoop();
    virtual void exitMainLoop();
    
    // Timer
    virtual int getMilliseconds();
    
protected:
    virtual void resetUpdateIntervals();

private:
    void handleEvent(const SDL_Event &evt);
    void keyDownEvent(const SDL_KeyboardEvent &kbd);
    void keyUpEvent(const SDL_KeyboardEvent &kbd);

    void initKeyTable(); // called during ctor

    bool windowed;
    bool has_focus;
    bool timer_set;         // is the SDL timer running
    SDL_TimerID timer_id;
    bool update_pending;    // set this true when an update is wanted.
    bool render_pending;    // set this true when a render is wanted.

    map<SDLKey, Keycode> keytable; // (a hash table might be better, or even a perfect hash.)

    vector<DisplayMode> display_modes;
};

#endif      // SDL_APP2D_HPP
