/*
 * binary_io.hpp
 *
 * Read/write data in binary onto streams. Make sure stream was opened in binary mode!
 *
 * these functions are a bit low level ... in particular we make assumptions about
 * sizes of integral types and endianness.
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef BINARY_IO_HPP
#define BINARY_IO_HPP

#include <iosfwd>
#include <string>

// Bytes, words (16-bit) and longs (32-bit).
signed char BinReadByte(std::istream &);
short int BinReadWord(std::istream &);
int BinReadLong(std::istream&);

void BinWriteByte(std::ostream&, signed char);
void BinWriteWord(std::ostream&, short int);
void BinWriteLong(std::ostream&, int);

unsigned char BinReadUbyte(std::istream &);
unsigned short int BinReadUword(std::istream &);
unsigned int BinReadUlong(std::istream &);

void BinWriteUbyte(std::ostream &, unsigned char);
void BinWriteUword(std::ostream &, unsigned short int);
void BinWriteUlong(std::ostream &, unsigned int);

// floats (assumed 32-bit)
float BinReadFloat(std::istream &);
void BinWriteFloat(std::ostream &, float);

// strings
std::string BinReadString(std::istream&);
void BinWriteString(std::ostream&, const std::string &);


// These write a "compressed" number format where we use bit 7 to say whether there will
// be another byte. best for small numbers.
// -64..+63 take 1 byte                   (2^6)
// -8192..+8191 take 2 bytes              (2^13)
// -1048576..+1048575 take 3 bytes        (2^20)
// -268435456..+268435455 take 4 bytes    (2^28)
int BinReadInt(std::istream&);
void BinWriteInt(std::ostream&, int);


#endif  // BINARY_IO_HPP
